/* ALGEN.C -- Public Domain -- By Richard Butler -- EUROPA Software 1992

    This code is public domain. Do with it what you please.
    All of the code is ANSI C and should compile with little or no change
    on any ANSI C compiler.

    This code carries no warranties, whether expressed or implied.

    Thanks to Harvey Parisien for his HLIST program upon which ALGEN is based.
    ALGEN's .cfg format and file list output are nearly identical to HLIST.

    Thanks to Erik Vanriper for releasing the source for his FILELIST program.
    It was a very useful reference for writing ALGEN.
*/

#include <stdio.h>
#include <time.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

char *whitespace = " \t\n\r";

main(int argc, char *argv[])
{
    char system_name[40];
    char header_file[80];
    int new_days;
    char list_file[80];
    char new_list_file[80];
    struct xx {
        char path[80];
        char desc[80];
        char files_bbs[80];
    };
    struct xx *area[256];
    struct stat info;
    struct tm current_date;
    struct tm new_date;
    struct tm *tm_ptr;
    time_t current_time;
    time_t new_time;
    unsigned area_count = 0;
    unsigned new_area_count = 0;
    unsigned total_count = 0;
    unsigned total_new_count = 0;
    unsigned long area_bytes = 0;
    unsigned long new_area_bytes = 0;
    unsigned long total_bytes = 0;
    unsigned long total_new_bytes = 0;
    FILE *config_file_p;
    FILE *header_file_p;
    FILE *list_file_p;
    FILE *new_list_file_p;
    FILE *fp;
    char config_file[80] = "ALGEN.CFG";     /* Default config file */
    int i = 0;
    size_t len;
    char io_buf[256];
    char *ptr;
    char *ws_ptr;
    int num_areas;
    char file_path[128];
    char file_name[13];
    unsigned char new;
    char *fd_ptr;
    int length;
    char stripflags;

    printf("\nķ");
    printf("\n ALGEN v1.00 - by Richard Butler - EUROPA Software          Fidonet 1:396/61 ");
    printf("\nĽ");
    fflush(stdout);
    for(i=0; i < 256; i++) {
        if((area[i] = malloc(sizeof(struct xx))) == NULL) {
            printf("\n\nNot enough memory!  Exiting....");
            exit(255);
        }
    }
    current_time = time(NULL);
    tm_ptr = localtime(&current_time);
    current_date = *tm_ptr;
    if(argc > 1)            /* assume that argv[1] is a valid config file */
        strcpy(config_file, argv[1]);
    if((config_file_p = fopen(config_file,"r")) == NULL) {
        printf("\n\nError opening %s!  Exiting....",config_file);
        exit(255);
    }

    /* Parse config file */

    printf("\n\nReading %s....",config_file);
    fflush(stdout);
    do {                                        /* get system name */
        fgets(io_buf,256,config_file_p);
    } while(io_buf[0] == ';');
    strncpy(system_name,io_buf,39);
    if(ptr = strchr(system_name,'\n'))
        *ptr = '\0';
    do {                                        /* get header file path */
        fgets(io_buf,256,config_file_p);
    } while(io_buf[0] == ';');
    strncpy(header_file,io_buf,79);
    if(ptr = strchr(header_file,'\n'))
        *ptr = '\0';
    do {                                        /* get new days */
        fgets(io_buf,256,config_file_p);
    } while(io_buf[0] == ';');
    new_days = atoi(io_buf);
    new_time = (current_time - (new_days * 86400));
    tm_ptr = localtime(&new_time);
    new_date = *tm_ptr;
    do {                                        /* get file list path */
        fgets(io_buf,256,config_file_p);
    } while(io_buf[0] == ';');
    strncpy(list_file,io_buf,79);
    if(ptr = strchr(list_file,'\n'))
        *ptr = '\0';
    do {                                        /* get new file list path */
        fgets(io_buf,256,config_file_p);
    } while(io_buf[0] == ';');
    strncpy(new_list_file,io_buf,79);
    if(ptr = strchr(new_list_file,'\n'))
        *ptr = '\0';
    do {                                        /* get STRIPFLAGS keyword */
        fgets(io_buf,256,config_file_p);
    } while(io_buf[0] == ';');
    if(ptr = strchr(io_buf,'\n'))
        *ptr = '\0';
    for(i = 0; io_buf[i] != '\0'; i++)
        io_buf[i] = toupper(io_buf[i]);
    stripflags = !(strcmp("STRIPFLAGS",io_buf));

    /* get files.bbs paths and area descriptions */

    i = 0;
    while(fgets(io_buf,256,config_file_p) != NULL) {
        if(io_buf[0] == ';')
            continue;
        strncpy(area[i]->path,io_buf,79);
        len = strcspn(area[i]->path,whitespace);
        area[i]->path[len] = '\0';
        if(area[i]->path[len - 1] != '\\')
            strncat(area[i]->path,"\\",79 - len);
        if(ptr = strchr(io_buf, ' ')) {
            ptr += (strspn(ptr,whitespace));
            strcpy(area[i]->desc," ");
            strncat(area[i]->desc,ptr,77);
        }
        if(ptr = strchr(area[i]->desc, '^')) {
            *ptr++ = ' ';
            *ptr = '\0';
            if(ptr = strchr(io_buf, '^'))
                strncpy(area[i]->files_bbs,++ptr,79);
            if(ptr = strchr(area[i]->files_bbs, '\n'))
                *ptr = '\0';
        }
        else if(ptr = strchr(area[i]->desc, '\n')) {
            *ptr++ = ' ';
            *ptr = '\0';
            strcpy(area[i]->files_bbs,area[i]->path);
            strncat(area[i]->files_bbs,"FILES.BBS",78 - len);
        }
        num_areas = ++i;
    }
    fclose(config_file_p);
    printf("\n\nProcessing....\n");
    fflush(stdout);

    /* open output files */

    if((list_file_p = fopen(list_file,"w")) == NULL) {
        printf("\n\nError opening %s!  Exiting....",list_file);
        exit(255);
    }
    if((new_list_file_p = fopen(new_list_file,"w")) == NULL) {
        printf("\n\nError opening %s!  Exiting....",new_list_file);
        exit(255);
    }
    fprintf(list_file_p,"\n                              F I L E    L I S T\n");
    fprintf(list_file_p,"\n %-53s%s",system_name,ctime(&current_time));
    fprintf(list_file_p,"ķ\n");
    fprintf(list_file_p,"        This is a complete list of all files available on this system        \n");
    fprintf(list_file_p,"Ľ\n");
    if((header_file_p = fopen(header_file,"r")) != NULL) {
        while(fgets(io_buf,256,header_file_p) != NULL)
            fputs(io_buf,list_file_p);
        fclose(header_file_p);
    }
    else {
        printf("\nError opening %s!  Continuing....",header_file);
        fflush(stdout);
    }
    fprintf(list_file_p,"ķ\n");
    sprintf(io_buf,"   File dates followed by a * are < than %d days old, which is >= %.2d-%.2d-%d",new_days,(new_date.tm_mon + 1),new_date.tm_mday,new_date.tm_year);
    strcat(io_buf,"          ");
    io_buf[78] = '';
    io_buf[79] = '\0';
    fprintf(list_file_p,"%s\n",io_buf);
    fprintf(list_file_p,"                                                                             \n");
    fprintf(list_file_p," ALGEN v1.00 - by Richard Butler - EUROPA Software          Fidonet 1:396/61 \n");
    fprintf(list_file_p,"Ľ\n");
    fprintf(new_list_file_p,"\n                              F I L E    L I S T\n");
    fprintf(new_list_file_p,"\n %-53s%s",system_name,ctime(&current_time));
    fprintf(new_list_file_p,"ķ\n");
    fprintf(new_list_file_p,"         This is a list of all 'NEW' files available on this system          \n");
    fprintf(new_list_file_p,"                                                                             \n");
    sprintf(io_buf,"             Files are < than %d days old, which is >= %.2d-%.2d-%d",new_days,(new_date.tm_mon + 1),new_date.tm_mday,new_date.tm_year);
    strcat(io_buf,"               ");
    io_buf[78] = '';
    io_buf[79] = '\0';
    fprintf(new_list_file_p,"%s\n",io_buf);
    fprintf(new_list_file_p,"                                                                             \n");
    fprintf(new_list_file_p," ALGEN v1.00 - by Richard Butler - EUROPA Software          Fidonet 1:396/61 \n");
    fprintf(new_list_file_p,"Ľ\n");

    /* process files.bbs */

    for(i = 0 ; i < num_areas ; i++) {
        area_count = 0;
        new_area_count = 0;
        area_bytes = 0;
        new_area_bytes = 0;
        strcpy(io_buf,"");
        strcat(io_buf,area[i]->desc);
        strcat(io_buf,"");
        fprintf(list_file_p,"\n%.79s\n",io_buf);
        fprintf(new_list_file_p,"\n%.79s\n",io_buf);
        if((fp = fopen(area[i]->files_bbs,"r")) == NULL) {
            printf("\nError opening %s!  Continuing....",area[i]->files_bbs);
            fflush(stdout);
        }
        else {
            printf("\n\t%s",area[i]->desc);
            fflush(stdout);
            while(fgets(io_buf,256,fp) != NULL) {
                if(io_buf[0] == '-' || io_buf[0] == ' ' || io_buf[0] == '\n') {
                    if(ptr = strchr(io_buf, '\n'))
                        *ptr = '\0';
                    io_buf[79] = '\0';
                    fprintf(list_file_p,"\n%s",io_buf);
                }
                else {
                    strncpy(file_name,io_buf,12);
                    if(ptr = strpbrk(file_name,whitespace))
                        *ptr = '\0';
                    strcpy(file_path,area[i]->path);
                    strcat(file_path,file_name);
                    if(stat(file_path,&info) == -1)
                        fprintf(list_file_p,"\n%-12s (stored offline) ",file_name);
                    else {
                        tm_ptr = localtime(&info.st_mtime);
                        fprintf(list_file_p,"\n%-12s %7ld %.2d-%.2d-%d",file_name,info.st_size,(tm_ptr->tm_mon + 1),tm_ptr->tm_mday,tm_ptr->tm_year);
                        area_count++;
                        total_count++;
                        area_bytes += info.st_size;
                        total_bytes += info.st_size;
                        if(new = ((current_time / 86400) - (info.st_mtime / 86400)) <= new_days) {
                            fprintf(new_list_file_p,"\n%-12s %7ld %.2d-%.2d-%d",file_name,info.st_size,(tm_ptr->tm_mon + 1),tm_ptr->tm_mday,tm_ptr->tm_year);
                            fprintf(list_file_p,"*");
                            fprintf(new_list_file_p,"*");
                            new_area_count++;
                            total_new_count++;
                            new_area_bytes += info.st_size;
                            total_new_bytes += info.st_size;
                        }
                        else
                            fprintf(list_file_p," ");
                    }
                    if(fd_ptr = strchr(io_buf, ' ')) {
                        fd_ptr += (strspn(fd_ptr,whitespace));
                        if(stripflags && (fd_ptr[0] == '/'))
                            fd_ptr = strpbrk(fd_ptr,whitespace);
                        if(ptr = strtok(fd_ptr,whitespace)) {
                            length = 30;
                            do {
                                if((length + strlen(ptr) + 1) < 80) {
                                    fprintf(list_file_p," %s",ptr);
                                    if(new)
                                        fprintf(new_list_file_p," %s",ptr);
                                    length += (strlen(ptr) + 1);
                                }
                                else {
                                    fprintf(list_file_p,"\n                              ");
                                    fprintf(list_file_p," %s",ptr);
                                    if(new) {
                                        fprintf(new_list_file_p,"\n                              ");
                                        fprintf(new_list_file_p," %s",ptr);
                                    }
                                    length = 31 + strlen(ptr);
                                }
                                ptr = strtok('\0',whitespace);
                            } while(ptr);
                        }
                    }
                }
            }
            fclose(fp);
        }
        fprintf(list_file_p,"\n\n  ** %d files in this area (%ld bytes)\n\n",area_count,area_bytes);
        fprintf(new_list_file_p,"\n\n  ** %d new files in this area (%ld bytes)\n\n",new_area_count,new_area_bytes);
    }

    /* finish things up */

    fprintf(list_file_p,"\n %-53s%s",system_name,ctime(&current_time));
    fprintf(list_file_p,"ķ\n");
    sprintf(io_buf," There are a total of %d files on this system (%ld bytes)",total_count,total_bytes);
    strcat(io_buf,"                               ");
    io_buf[78] = '';
    io_buf[79] = '\0';
    fprintf(list_file_p,"%s\n",io_buf);
    fprintf(list_file_p,"                                                                             \n");
    fprintf(list_file_p," ALGEN v1.00 - by Richard Butler - EUROPA Software          Fidonet 1:396/61 \n");
    fprintf(list_file_p,"Ľ\n");
    fprintf(new_list_file_p,"\n %-53s%s",system_name,ctime(&current_time));
    fprintf(new_list_file_p,"ķ\n");
    sprintf(io_buf," There are a total of %d new files on this system (%ld bytes)",total_new_count,total_new_bytes);
    strcat(io_buf,"                               ");
    io_buf[78] = '';
    io_buf[79] = '\0';
    fprintf(new_list_file_p,"%s\n",io_buf);
    fprintf(new_list_file_p,"                                                                             \n");
    fprintf(new_list_file_p," ALGEN v1.00 - by Richard Butler - EUROPA Software          Fidonet 1:396/61 \n");
    fprintf(new_list_file_p,"Ľ\n");
    fclose(list_file_p);
    fclose(new_list_file_p);
    printf("\n\n....Done!\n");
}
